# inspired from
# https://github.com/readthedocs/commonmark.py/blob/master/commonmark/render/html.py
# https://github.com/readthedocs/commonmark.py/blob/master/commonmark/render/renderer.py

import re
from builtins import str
import fitz
from pathlib import Path


from . import log
from . import style
from . import font
from . import properties
from .headfoot import Header, Footer


fontSize = 10  # choose font size of text
headingfontSizes = [18, 16, 14, 12, 10, 10, 10, 10]
lineheight = fontSize * 1.2  # line height is 20% larger
margin = 72

# For images, we are looking for a pattern in the alt text,
# e.g. Some text { width =31% }
# grab the number for image scale, and grab the remainder for the caption.
# This should be "compatible" with other Markdown variants.
# e.g. Pandoc uses alt text for image caption.
imageWidthRe = re.compile(r"(.*){.*width\s*=\s*(\d+)%.*}")
# https://regex101.com/r/8zgA9P/2


class PdfRenderer:
    def __init__(self, pdf):
        self.list_data = list()  # to store the states of ordered/unordered list
        # TODO: put this in render() Directory containing markdown (and images)
        self.indir = None
        self.pdf = pdf
        self.doc = fitz.open()
        self.toc = []
        self.currentPage = None

        global width, height
        width, height = fitz.paper_size(properties.paperSize)  # choose paper format

    def __del__(self):

        # Close the file.  If an exception occured, the attribute might not be present,
        # so check first.
        if hasattr(self, "doc"):
            # If still None, we haven't processed an ast
            if self.doc.pageCount:
                for i in range(len(self.toc)):
                    log.debug(f"{i}, {self.toc[i]}")
                try:
                    self.doc.setToC(self.toc)
                except ValueError as e:
                    log.exception("Bad heading level.  More information:")
                    lastspace = str(e).rfind(" ")
                    log.info(self.toc[int(str(e)[lastspace:])])
                self.doc.setMetadata(properties.document)
                try:
                    self.doc.save(str(self.pdf), garbage=4, deflate=True)
                except RuntimeError as e:
                    log.exception(e)
                self.doc.close()
            else:
                log.info("No pages to save")

    def render(self, ast, inputFile):
        self.infile = Path(inputFile).absolute()
        self.indir = Path(inputFile).parent.resolve()
        for node, entering in ast.walker():
            getattr(self, node.t)(node, entering)

        return

    def escape(self, text):
        return text

    def tag(self, name, attrs=None, selfclosing=None):
        return

    def text(self, node, entering=None):
        # TODO: what to do with text in image "alt" field?
        # we are currently using it for non-standard "width"
        # and remaining text could be printed.  For now, drop it.
        if node.parent is not None:
            if node.parent.t == "image":
                self.cr("caption")
                self.printCentre(node.literal)
                self.cr("caption")
                return
        line = node.literal
        self.printLine(line)

    # print a line that may break
    def printLine(self, line):
        sty = style.currentStyle()
        fontName = sty.font.name
        fontSize = sty.fontsize
        lineWidth = fitz.get_text_length(line, fontName, fontSize)
        budget = width - margin - self.indent - self.insertPoint.x
        if lineWidth < budget:
            self.printSegment(line)
        else:
            prefix = line
            suffix = line
            while True:
                index = line.rfind(" ", 0, len(prefix))
                if index == -1:  # No spaces found.  Print it on the next line
                    self.cr(chr(0xAC))
                    self.printSegment(suffix)
                    break
                prefix = line[:index]
                prefixWidth = fitz.get_text_length(
                    prefix, fontname=fontName, fontsize=fontSize
                )
                if prefixWidth > budget:  # Still too large.  Chop again.
                    continue
                else:
                    self.printSegment(prefix)
                    suffix = line[index + 1 :]
                    # Recurse for the remainder.  It might be too long as well.
                    self.cr(chr(0xAC))
                    self.printLine(suffix)
                    break

    # Softbreaks are just CRs in the input, within paragraph
    # it becomes a space.
    def softbreak(self, node=None, entering=None):
        sty = style.currentStyle()
        fontName = sty.font.name
        fontSize = sty.fontsize
        lineWidth = fitz.get_text_length(" ", fontname=fontName, fontsize=fontSize)
        budget = width - margin - self.insertPoint.x
        if lineWidth < budget:
            self.printSegment(" ")
        else:
            pass  # We're about to to a hard break anyway.
        # self.currentPage.insertText(self.insertPoint, " ", fontSize=fontSize)
        # self.insertPoint.x += fitz.get_text_length(" ", fontSize=fontSize)

    def linebreak(self, node=None, entering=None):
        self.cr("linebreak")

    def link(self, node, entering):
        if entering:
            self.linkDestination = node.destination  # TODO: deal with "#fragments"
        else:
            # the link on the page can be split over several words & lines
            # So we need to create the clickable areas.

            if ":" in self.linkDestination:
                for rect in self.linkRects:
                    newLink = {
                        "kind": fitz.LINK_URI,
                        "from": rect,  # the area on the page that is "clickable"
                        "page": None,
                        "to": None,
                        "file": None,
                        "uri": self.linkDestination,
                        "xref": None,
                    }
                    self.currentPage.insertLink(newLink)
                self.linkRects.clear()
            else:
                filename = Path(self.indir) / self.linkDestination
                try:

                    # This is ugly but whatever
                    pin = fitz.Point(
                        self.insertPoint.x, self.insertPoint.y - lineheight
                    )
                    self.currentPage.addFileAnnot(
                        pin,
                        buffer=open(filename, mode="rb").read(),
                        # This goes in Description  TODO: file issue with PyMuPDF
                        filename=node.first_child.literal,
                        ufilename=filename.name,
                        # Not working.  See TODO: file issue with PyMuPDF
                        desc="deschere",
                        icon="Paperclip",
                    )

                    # This doesn't work but keeping as comments for future reference.
                    # (Linking to an embedded PDF only works on PDF >= 1.6)
                    # rc = self.doc.embeddedFileAdd(
                    #     self.linkDestination,  # entry identifier
                    #     buffer=open(filename, mode="rb").read(),
                    #     filename="NLTEST",  # Optional
                    #     desc=node.first_child.literal,  # Optional
                    # )
                    # for rect in self.linkRects:
                    #     newLink = {
                    #         "kind": fitz.LINK_GOTO,  # See PyMuPDF 6.11.2
                    #         "from": rect,  # the area on the page that is "clickable"
                    #         "page": 1,
                    #         # "to": , # For LINK_GOTOR, defaults to Point(0,0), OK
                    #         "file": self.linkDestination,
                    #         # "uri": None,  # Only for URI, otherwise ignored
                    #         # "xref": None,  # OK
                    #     }
                    #     self.currentPage.insertLink(newLink)

                except FileNotFoundError as err:
                    self.markdownError(node, f"{node.destination}: {err.strerror}")

            self.linkDestination = None

    def findSource(self, node):
        if node.sourcepos:
            return node.sourcepos[1][0]
        else:
            return self.findSource(node.parent)

    def markdownError(self, node, msg):
        log.error(f"{self.infile}:{self.findSource(node)}: {msg}")

    def image(self, node, entering):
        from . import image

        if entering:
            # TODO: consider using the alt attribute to pass the rectangle
            # dimensions/alignment/etc. https://www.w3schools.com/tags/att_img_alt.asp
            # This means the next node would need to be "peaked" at, since that's how
            # commonmark parses it.
            # Also, the text callback would have to check if it's the child of an
            # image node, OR, this function would
            # jump to the next next child somehow (return??  ) /TODO

            try:
                widthAvailable = width - 2 * margin
                imagefile = Path(self.indir) / node.destination
                imageW, imageH = image.get_image_size(str(imagefile))
                log.debug(f"{imagefile}, {imageW}, {imageH}")
                imageRatio = imageW / imageH
                rectWidth, rectHeight = imageW, imageH  # default w,h

                # Use all available width if desired, otherwise centre image.
                if rectWidth > widthAvailable:
                    rectWidth = widthAvailable
                    rectHeight = rectWidth / imageRatio
                    rect = fitz.Rect(
                        self.insertPoint,
                        width - margin,
                        self.insertPoint.y + rectHeight,
                    )
                else:
                    self.insertPoint.x = (width - rectWidth) / 2
                    rect = fitz.Rect(
                        self.insertPoint,
                        self.insertPoint.x + rectWidth,
                        self.insertPoint.y + rectHeight,
                    )

                # If the alt field (next node) has width specified, use it
                # This is the width of the image relative to printable area
                if node.first_child:
                    desiredWidth = imageWidthRe.match(node.first_child.literal)
                    if desiredWidth:
                        node.first_child.literal = imageWidthRe.sub(
                            desiredWidth.group(1), node.first_child.literal
                        )
                        rectScale = float(desiredWidth.group(2)) / 100
                        rectWidth = widthAvailable * rectScale
                        rectHeight = rectWidth / imageRatio
                        self.insertPoint.x = (width - rectWidth) / 2
                        rect = fitz.Rect(
                            self.insertPoint,
                            self.insertPoint.x + rectWidth,
                            self.insertPoint.y + rectHeight,
                        )

                # If we're running out of room start a new page
                if rectHeight > height - margin - self.insertPoint.y:
                    self.newPage()
                    self.insertPoint.x = (width - rectWidth) / 2
                    rect = fitz.Rect(
                        self.insertPoint,
                        self.insertPoint.x + rectWidth,
                        self.insertPoint.y + rectHeight,
                    )

                self.currentPage.insertImage(
                    rect, filename=str(imagefile), keep_proportion=True
                )
                self.insertPoint.y += rectHeight
            except FileNotFoundError as err:
                self.markdownError(node, f"{node.destination}: {err.strerror}")

        # node.title

    def emph(self, node, entering):
        if entering:
            style.push(italic=True)
        else:
            style.pop()

    def strong(self, node, entering):
        if entering:
            style.push(bold=True)
        else:
            style.pop()

    def paragraph(self, node, entering):
        if entering:
            if node.parent is not None:
                if node.parent.t == "block_quote":
                    return
            if node.parent.parent is not None:
                if node.parent.parent.t == "list":
                    # if grandparent.list_data["tight"]:
                    # TODO maybe deal with tight/loose lists
                    return
            self.crHalfLine("paragraphhalf+")
        else:
            # if node.parent.parent is None:  #
            #     self.cr("p-")
            # elif node.parent.parent.t == "list":
            self.cr("p-")

    def heading(self, node, entering):
        global fontSize

        if not hasattr(node.first_child, "literal"):
            self.markdownError(node, "Empty heading")
            return

        if entering:
            style.push(fontname=font.HELVETICA, fontsize=headingfontSizes[node.level])
            lineheight = style.currentStyle().lineheight
            if self.insertPoint.y > margin + lineheight:
                self.cr("H+")
            if node.level == 1:
                properties.setSection(node.first_child.literal)
        else:
            self.toc.append(
                [
                    node.level,
                    node.first_child.literal,
                    self.doc.pageCount,
                    self.insertPoint.y - headingfontSizes[node.level],
                ]
            )
            self.cr("H-")
            style.pop()
            lineheight = style.currentStyle().lineheight

    def code(self, node, entering):
        style.push(fontname=font.COURIER)
        self.printLine(node.literal)
        style.pop()

    def code_block(self, node, entering):
        if entering:
            style.push(fontname=font.COURIER)
            self.crHalfLine("codehalf+")
            self.indent += 32
            self.insertPoint.x += 32
            for line in node.literal.split("\n"):
                if len(line):
                    self.printLine(line)
                    self.cr("")

            # else: # doesn't get called with entering = False?
            self.indent -= 32
            self.insertPoint.x -= 32
            self.crHalfLine("codehalf-")
            style.pop()

    def thematic_break(self, node, entering):
        # attrs = self.attrs(node)
        pntFrom = fitz.Point(self.insertPoint.x, self.insertPoint.y - lineheight / 2)
        pntTo = fitz.Point(width - margin, pntFrom.y)
        shape = self.currentPage.newShape()
        shape.drawLine(pntFrom, pntTo)
        shape.finish()
        shape.commit()
        self.cr("")

    def block_quote(self, node, entering):
        # attrs = self.attrs(node)
        if entering:
            self.crHalfLine("bqhalf+")
            self.indent += 32
            self.insertPoint.x += 32

        else:
            self.indent -= 32
            self.insertPoint.x -= 32
            self.crHalfLine("bqhalf-")

    def list(self, node, entering):
        # node.list_data
        if entering:
            self.list_data.append(node.list_data)
            self.crHalfLine("listhalf+")
            self.indent += 16
            self.insertPoint.x = margin + self.indent
        else:
            self.list_data.pop()
            self.crHalfLine("listhalf-")
            self.indent -= 16
            self.insertPoint.x = margin + self.indent

    def item(self, node, entering):
        # attrs = self.attrs(node)
        if entering:
            if self.list_data[-1]["type"] == "ordered":
                self.printSegment(f" {node.list_data['start']} ")
            else:
                style.push(fontname=font.ZAPFDINGBATS)
                self.printSegment("l")  # Bullet:
                #  https://help.adobe.com/en_US/framemaker/2015/using/using-framemaker-2015/Appendix/frm_character_sets_cs/frm_character_sets_cs-5.htm
                style.pop()
            self.indent += 16
            self.insertPoint.x = margin + self.indent

        else:
            self.indent -= 16
            self.insertPoint.x = margin + self.indent

    # From spec: Text between < and > that looks like an HTML tag is parsed
    # as a raw HTML tag and will be rendered in HTML without escaping. Tag
    # and attribute names are not limited to current HTML tags, so custom
    # tags (and even, say, DocBook tags) may be used.

    def html_inline(self, node, entering):
        self.code(node, entering)  # TODO maybe warn user?

    def html_block(self, node, entering):
        self.code_block(node, entering)  # TODO maybe warn user?

    def custom_inline(self, node, entering):
        self.code(node, entering)  # TODO maybe warn user?

    def custom_block(self, node, entering):
        self.code(node, entering)  # TODO maybe warn user?

    # Helper methods #

    def cr(self, note):
        # self.currentPage.insertText(self.insertPoint, note, fontsize=3)
        lineheight = style.currentStyle().lineheight

        self.insertPoint.x = margin + self.indent
        self.insertPoint.y += lineheight
        if self.insertPoint.y > height - margin:
            self.newPage()

    def crHalfLine(self, note="half"):
        # self.currentPage.insertText(self.insertPoint, note, fontsize=3)
        self.insertPoint.x = margin + self.indent
        self.insertPoint.y += lineheight / 2
        if self.insertPoint.y > height - margin:
            self.newPage()

    def printSegment(self, line):
        sty = style.currentStyle()
        fontName = sty.font.name
        fontSize = sty.fontsize
        lineWidth = fitz.get_text_length(line, fontname=fontName, fontsize=fontSize)

        # self.currentPage.insertText(
        #     self.insertPoint,
        #     str(self.insertPoint.x),
        #     fontname="cobo",
        #     fontsize=fontSize * 2,
        # )
        log.debug(f"printSegment: {line}")
        self.currentPage.insertText(
            self.insertPoint, line, fontname=fontName, fontsize=fontSize
        )
        if self.linkDestination:
            self.linkRects.append(
                fitz.Rect(
                    self.insertPoint.x,
                    self.insertPoint.y - lineheight,
                    self.insertPoint.x + lineWidth,
                    self.insertPoint.y,
                )
            )
        self.insertPoint.x += lineWidth

    def document(self, node, entering):
        if entering:
            self.indent = 0
            self.insertPoint = fitz.Point(margin, margin + lineheight)
            self.linkDestination = None
            self.linkRects = []

            self.currentPage = self.doc.newPage(-1, width, height)
            style.push(fontname=font.TIMES, fontsize=10, indent=0)
        else:
            self.finishPage()
            style.pop()  # We should be done anyway

    def finishPage(self):
        link = self.currentPage.firstLink
        while link:
            link.setBorder({"width": 0.5, "style": "U"})
            link = link.next

        # Generate Header and Footer
        properties.page = self.doc.pageCount

        style.push(fontname=font.HELVETICA, fontsize=8, indent=0)
        if Header.enabled:
            self.insertPoint.y = margin / 2 + lineheight / 2
            left = Header.left(properties)
            middle = Header.mid(properties)
            right = Header.right(properties)
            self.printLeft(left)
            self.printCentre(middle)
            self.printRight(right)

            pntFrom = fitz.Point(margin, 0.75 * margin)
            pntTo = fitz.Point(width - margin, 0.75 * margin)
            self.currentPage.drawLine(pntFrom, pntTo, width=1)

        if Footer.enabled:
            self.insertPoint.y = height - (margin / 2) + lineheight / 2
            left = Footer.left(properties)
            middle = Footer.mid(properties)
            right = Footer.right(properties)
            self.printLeft(left)
            self.printCentre(middle)
            self.printRight(right)

            pntFrom = fitz.Point(margin, height - (0.75 * margin))
            pntTo = fitz.Point(width - margin, height - (0.75 * margin))
            self.currentPage.drawLine(pntFrom, pntTo, width=1)
        style.pop()

    def newPage(self):
        self.finishPage()
        self.currentPage = self.doc.newPage(-1, width, height)
        self.insertPoint = fitz.Point(margin + self.indent, margin + lineheight)

    def printLeft(self, line):
        sty = style.currentStyle()
        self.insertPoint.x = margin
        self.currentPage.insertText(
            self.insertPoint, line, fontname=sty.font.name, fontsize=sty.fontsize
        )

    def printCentre(self, line):
        sty = style.currentStyle()
        lineWidth = fitz.get_text_length(
            line, fontname=sty.font.name, fontsize=sty.fontsize
        )
        self.insertPoint.x = width / 2 - lineWidth / 2
        self.currentPage.insertText(
            self.insertPoint, line, fontname=sty.font.name, fontsize=sty.fontsize
        )

    def printRight(self, line):
        sty = style.currentStyle()
        lineWidth = fitz.get_text_length(
            line, fontname=sty.font.name, fontsize=sty.fontsize
        )
        self.insertPoint.x = width - margin - lineWidth
        self.currentPage.insertText(
            self.insertPoint, line, fontname=sty.font.name, fontsize=sty.fontsize
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This is the entry point for the command-line interface (CLI) application.

Itcan be used as a handy facility for running the task from a command line.

    To learn more about Click visit the
    `project website <http://click.pocoo.org/5/>`_.  There is also a very
    helpful `tutorial video <https://www.youtube.com/watch?v=kNke39OZ2k0>`_.

"""
import click
import glob

from mdpdf.converter import Converter
from mdpdf.headfoot import Header, Footer
from mdpdf.properties import setTitle, setSubject, setAuthor, setKeywords, setPaperSize
from mdpdf import log

# TODO: consider pip install click-config-file


@click.command()
@click.option(
    "--output", "-o", metavar="FILE", required=True, help="Destination for file output."
)
@click.option("--header", "-h", metavar="<template>", help="Sets the header template.")
@click.option("--footer", "-f", metavar="<template>", help="Footer template.")
@click.option("--title", "-t", default="", help="PDF title.")
@click.option("--subject", "-s", default="", help="PDF subject.")
@click.option("--author", "-a", default="", help="PDF author.")
@click.option("--keywords", "-k", default="", help="PDF keywords.")
@click.option(
    "--paper",
    "-p",
    default="letter",
    type=click.Choice(["letter", "A4"], case_sensitive=False),
    help="Paper size (default letter).",
)
@click.version_option()
@click.argument("inputs", nargs=-1)
def cli(
    output: str,
    header: str,
    footer: str,
    title: str,
    subject: str,
    author: str,
    keywords: str,
    paper: str,
    inputs,
):
    """Convert Markdown to PDF.

    \b
    For options below, <template> is a quoted, comma-
    delimited string, containing the left, centre,
    and right, header/footer fields. Format is

      "[left],[middle],[right]"

    \b
    Possible values to put here are:
      - Empty string
      - Arbitrary text
      - {page} current page number
      - {header} current top-level body text heading
      - {date} current date"""

    ctx = click.get_current_context()
    if not output:
        ctx.fail("No output specified.")

    try:
        if header:
            Header.setFmt(header)
        if footer:
            Footer.setFmt(footer)
    except Exception as e:
        ctx.fail(f"{e} in header/footer template.")

    if title:
        setTitle(title)

    if author:
        setAuthor(author)

    if subject:
        setSubject(subject)

    if keywords:
        setKeywords(keywords)

    if paper:
        setPaperSize(paper)

    if inputs:
        log.init()
        globlist = []
        for i in inputs:
            log.debug(i)
            matches = glob.glob(i)
            if matches:
                globlist.extend(matches)
            else:
                ctx.fail(f"File not found: {i}.")
        converter = Converter(output)
        converter.convert(globlist)
    else:
        ctx.fail("No input specified.")


if __name__ == "__main__":
    import sys

    cli.main(sys.argv[1:])



import commonmark

from . import log
from .pdf_renderer import PdfRenderer


class Converter:
    def __init__(self, outputFileName):
        self.parser = commonmark.Parser()
        self.renderer = PdfRenderer(outputFileName)

    def convert(self, inputFileNames):
        for inputFile in inputFileNames:
            log.info(inputFile)
            mdFile = open(inputFile, "r", encoding="utf-8")
            entireFile = mdFile.read()
            ast = self.parser.parse(entireFile)
            self.renderer.render(ast, inputFile)



import fitz

# _BASE14 = fitz.Base14_fontdict

# Font attribute names
NAME = "fontname"
SIZE = "fontsize"
INDENT = "indent"
STRONG = "bold"
EMPHASIS = "italic"


COURIER = "cour"
TIMES = "tiro"
HELVETICA = "helv"
SYMBOL = "symb"
ZAPFDINGBATS = "zadb"

_BOLD = "bo"
_ITALIC = "it"
BOLD_ITALIC = "bi"


class Base14:
    def __init__(self, name):
        if name not in [COURIER, TIMES, HELVETICA, SYMBOL, ZAPFDINGBATS]:
            raise Exception("not valid Base14")

        self._name = name
        self._bold = False
        self._italic = False

    # def setModifier(self):
    #     if self._root not in [COURIER, TIMES, HELVETICA]:
    #         return  # TODO: raise?  symbol and zapf don't have modifiers
    #     if self._bold and self._italic:
    #         self._name = self._name[:2] + BOLD_ITALIC
    #     elif self._bold:
    #         self._name = self._name[:2] + _BOLD
    #     elif self._italic:
    #         self._name = self._name[:2] + _ITALIC
    #     else:
    #         self._name = self._root

    def setItalic(self, state):
        self._italic = state
        # self.setModifier()

    def setBold(self, state):
        self._bold = state
        # self.setModifier()

    @property
    def name(self):
        if self._name in [ZAPFDINGBATS, SYMBOL]:
            return self._name
        if self._name not in [COURIER, TIMES, HELVETICA]:
            return  # TODO: raise?  symbol and zapf don't have modifiers
        if self._bold and self._italic:
            name = self._name[:2] + BOLD_ITALIC
        elif self._bold:
            name = self._name[:2] + _BOLD
        elif self._italic:
            name = self._name[:2] + _ITALIC
        else:
            name = self._name
        return name



from . import font
import copy

_styleList = []
_styleNumber = 0


class Style:
    def __init__(self, **kwargs):
        if kwargs:
            self.setAttributes(**kwargs)

    def setAttributes(self, **kwargs):
        for var, val in kwargs.items():
            # log.info(f"{var}, {val}")
            if var == "fontname":
                self.font = font.Base14(val)
            elif var == "fontsize":
                self.fontsize = val
            elif var == "italic":
                self.font.setItalic(val)
            elif var == "bold":
                self.font.setBold(val)
            elif var == "indent":
                self.indent = val
            else:
                raise Exception("not valid Style attribute")

        self.lineheight = self.fontsize * 1.2


def push(**kwargs):
    if len(_styleList):
        newStyle = copy.deepcopy(currentStyle())
        newStyle.setAttributes(**kwargs)
    else:
        newStyle = Style(**kwargs)

    _styleList.append(newStyle)


def pop():
    global _styleNumber
    _styleNumber -= 1

    _styleList.pop()


def currentStyle():
    return _styleList[-1]


if __name__ == "__main__":

    push(fontname=font.HELVETICA, fontsize=12)
    style = currentStyle()
    push(indent=36)
    push(italic=True)
    style = currentStyle()
    pop()
    style = currentStyle()

    pass



from logging import *
from pathlib import Path

# set up logging to file - see previous section for more details
LONGFORMAT = (
    "%(levelname)8s: " "%(asctime)s: " "%(filename)20s: " "%(lineno)4d: " "%(message)s"
)
SHORTFORMAT = "%(message)s"

# Root logger gets everything.  Handlers defined below will filter it out...

getLogger("").setLevel(DEBUG)


def init(filename=Path("mdpdf.log")):
    filehandler = FileHandler(filename, mode="w", encoding="utf-8")
    filehandler.setLevel(DEBUG)
    filehandler.setFormatter(Formatter(LONGFORMAT))
    getLogger("").addHandler(filehandler)
    info(f"Logging to {filename.absolute()}")


# define a Handler which writes to sys.stderr
console = StreamHandler()
console.setLevel(INFO)
console.setFormatter(Formatter(SHORTFORMAT))
# add the handler to the root logger
getLogger("").addHandler(console)



import fitz
import datetime

# This dict is specific to pyMuPDF.
# Keys can only be : format, encryption, title, author, subject, keywords, creator, producer, creationDate, modDate

document = {
    "creationDate": fitz.get_pdf_now(),  # current timestamp
    "modDate": fitz.get_pdf_now(),  # current timestamp
    "creator": "Python mdfpdf package: https://pypi.org/project/mdpdf",
    "producer": "PyMuPDF library: https://pypi.org/project/PyMuPDF",
    "title": None,
    "subject": None,
    "author": None,
    "keywords": None,
}

# These properties are used for the header and footer
# these are available on the command-line switch --header and --footer
page = 1
heading = "some heading"
date = datetime.datetime.now().date()

paperSize = "letter"


def setPaperSize(text):
    global paperSize
    paperSize = text


def setSection(text):
    global heading
    heading = text


def setTitle(text):
    document["title"] = text


def setSubject(text):
    document["subject"] = text


def setAuthor(text):
    document["author"] = text


def setKeywords(text):
    document["keywords"] = text



#


class Base:
    enabled = False

    @classmethod
    def setFmt(cls, fmt):
        cls._left, cls._mid, cls._right = fmt.split(",")
        cls.enabled = True

    @classmethod
    def left(cls, m):
        return cls._left.format(**m.__dict__)

    @classmethod
    def mid(cls, m):
        return cls._mid.format(**m.__dict__)

    @classmethod
    def right(cls, m):
        return cls._right.format(**m.__dict__)


class Header(Base):
    pass


class Footer(Base):
    pass


if __name__ == "__main__":
    Header.setFmt("{date},blah,{page}")

    class MetaData(object):
        pass

    metaData = MetaData()
    metaData.date = "2020-02-15"
    metaData.page = 12
    metaData.header = "chapter 1"

    # metaData = {"date": date, "page": page, "header": header}

    l = Header.left(metaData)
    m = Header.mid(metaData)
    r = Header.right(metaData)

    metaData.page += 1
    r = Header.right(metaData)

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Python command line application to convert Markdown to PDF.

.. currentmodule:: mdpdf
.. moduleauthor:: Norman Lorrain <normanlorrain@gmail.com>
"""




# from  https://stackoverflow.com/questions/8032642/how-to-obtain-image-size-using-standard-python-class-without-using-external-lib


import struct
import imghdr


def get_image_size(fname):
    """Determine the image type of fhandle and return its size.
    from draco"""
    with open(fname, "rb") as fhandle:
        head = fhandle.read(24)
        if len(head) != 24:
            return
        if imghdr.what(fname) == "png":
            check = struct.unpack(">i", head[4:8])[0]
            if check != 0x0D0A1A0A:
                return
            width, height = struct.unpack(">ii", head[16:24])
        elif imghdr.what(fname) == "gif":
            width, height = struct.unpack("<HH", head[6:10])
        elif imghdr.what(fname) == "jpeg":
            try:
                fhandle.seek(0)  # Read 0xff next
                size = 2
                ftype = 0
                while not 0xC0 <= ftype <= 0xCF:
                    fhandle.seek(size, 1)
                    byte = fhandle.read(1)
                    while ord(byte) == 0xFF:
                        byte = fhandle.read(1)
                    ftype = ord(byte)
                    size = struct.unpack(">H", fhandle.read(2))[0] - 2
                # We are at a SOFn block
                fhandle.seek(1, 1)  # Skip `precision' byte.
                height, width = struct.unpack(">HH", fhandle.read(4))
            except Exception:  # IGNORE:W0703
                return
        else:
            return
        return width, height
